@@ -1,8 +1,9 @@
 from enum import Enum
+from typing import Union
 from urllib.parse import quote_plus
 
 from pymongo import MongoClient
-from pymongo.errors import ServerSelectionTimeoutError
+from pymongo.errors import ServerSelectionTimeoutError, OperationFailure
 
 MAX_SEV_SEL_DELAY = 5
 
@@ -44,7 +45,7 @@ def uri_connect(host: str = 'localhost', port: int = 27017, replicaset: str = No
     if mode == Mode.AUTO:
         raise ValueError('The mode AUTO is not supported directly by this function.'
                          ' You need to use connect() or connect_database() functions.')
-    user = quote_plus(user) if mode in [Mode.GSSAPI, Mode.Windows] else user
+    user = quote_plus(user) if user and mode in [Mode.GSSAPI, Mode.Windows] else user
     uri = f'mongodb://'
     if mode not in [Mode.AWS_Lambda, Mode.ECS_Container, Mode.EC2_Instance]:
         uri += f'{user}' if user else ''
@@ -156,12 +157,15 @@ def detect_connection(host: str = 'localhost', port: int = 27017, replicaset: st
     :param session_token: The session token to use.
     :return: The MongoDB client.
     """
+    error = None
     for mode in list(Mode)[2:]:
         try:
             client = connect_database(host, port, replicaset, database,
                                       user, password, mode, cert_key_file, ca_file, session_token)
-            client.server_info()
+            client.list_database_names()
             return client
         except ServerSelectionTimeoutError as e:
-            last_error = e
-        raise last_error
+            error = error if error else e
+        except OperationFailure as e:
+            error = OperationFailure('Authentication error')
+    raise error
